function [U, V, C1, D1, H, K] = krylov_ata_extended_expand(A, Ainv, U, V, C1, D1, H, K, k)

%KRYLOV_ATA_EXTENDED_EXPAND  Construct orthonormal basis for extended bidiagonalization subspace
% [U, V, C1, D1, H, K] = krylov_ata_extended_expand(A, Ainv, U, V, C1, D1, H, K, k)
%
% Ainv(x,0) should carry out A\x
% Ainv(x,1) should carry out A'\x
%
% See also KRYLOV_ATA_EXTENDED
%
% Revision date: November 14, 2014
% (C) Sarah Gaaf, Michiel Hochstenbach 2016

if nargin < 9 || isempty(k)
  k = 2;
end

m = size(V,2);

for j = m:(k+m-1)
  if mod(j,2) == 1
    [U(:,j), C(1:j,j)]     = rgs(mv(A,V(:,j),0), U);            % u = Av
    [V(:,j+1), D(1:j+1,j)] = rgs(mv(A,U(:,j),1), V);            % v = A'u
    C1(1:3,j) = C(j-2:j,j);  
    D1(1:3,j) = D(j-1:j+1,j);
  else
    [U(:,j), C(1:j,j)]     = rgs(mv(Ainv,V(:,j),1), U);         % u = A'\v   
    [V(:,j+1), D(1:j+1,j)] = rgs(mv(Ainv,U(:,j),0), V);         % v = A\u
    C1(1:3,j) = C(j-2:j,j);
    D1(1:3,j) = D(j-1:j+1,j);
  end
end

%-------------------------------------------------------------------------
% Construction of the matrices H and K
%-------------------------------------------------------------------------
H1 = zeros(k+m-1,k+m-1);
H1(1:m-1,1:m-1) = H;
H = H1;
K1 = zeros(k+m,k+m-1);
K1(1:m,1:m-1) = K;
K = K1;
for j = m:2:(k+m-1)
  H(j,:) = D(1:k+m-1,j)';
  K(j,j) = 1/C(j,j);
end
for j = (m+1):2:(k+m)
  H(j,j) = 1/C(j,j);
  K(:,j) = D(:,j);
end
