function [delta, k] = lanprob_delta(n, epsilon)

%LANPROB_DELTA  Delta needed in the probabilistic Lanczos method
% function [delta, k] = lanprob_delta(n, epsilon)
%
% Reference:
%   J.L.M. van Dorsselaer, M.E. Hochstenbach, H.A. van der Vorst
%     Computing probabilistic bounds for extreme eigenvalues
%       of symmetric matrices with the Lanczos method,
%     SIAM J. Matrix Anal. Appl. 22(3), pp. 837-852, 2000
%
% See also LANPROB
%
% Revision date: May 25, 2016
% (C) Michiel Hochstenbach 2016

if nargin < 1 || isempty(n)
  n = 1000;        % Dimension of matrix
end
if nargin < 2 || isempty(epsilon)
  epsilon = 0.01;  % Corresponds to probabilist bounds that hold with probability >= 99%
end

c = epsilon*beta((n-1)/2, 0.5) / 2;
f = @(d) integral(@(t) (cos(t)).^(n-2), 0, asin(d)) - c;

% delta should be very close to c
[delta, k] = bisection(f, 0, min([1.2*c 1]), 1000, 1e-14);
