function M = lu0(A)

%LU0  LU decomposition with some extras
% function M = lu0(A)
% M(x,t) carries out A\x (t=0) or A'\x (t=1)
%
% See also LU, LU_INV
%
% Revision date: May 27, 2014
% (C) Michiel Hochstenbach 2016

if ~issparse(A)
  [L,U,p] = lu(A, 'vector');
  M = @(x,t) lu_inv(x, L, U, p, t);
else
  [L,U,p,q,R] = lu(A, 'vector');
  q2(q) = 1:length(q);
  M = @(x,t) lu_inv(x, L, U, p, q, q2, R, t);
end


function y = lu_inv(x, L, U, p, q, q2, R, transp_flag)

%LU_INV  Matrix inverse times vector using LU information
% function y = lu_inv(x, L, U, p, q, q2, R, transp_flag)
%
% Makes use of by Tim Davis' cs_usolve and cs_ltsolve

if nargin < 6
  % LU of full matrix
  if nargin < 5 || isempty(q)
    transp_flag = 0;
  else
    transp_flag = q;
  end
elseif nargin < 8 || isempty(transp_flag)
  transp_flag = 0;
end

if nargin < 6
  if ~transp_flag
    y = cs_usolve(U, cs_lsolve(L, x(p,:)));
  else
    p2(p) = 1:length(p);
    y = cs_ltsolve(L, cs_utsolve(U, x));
    y = y(p2);
  end
else
  if ~transp_flag
    y = cs_usolve(U, cs_lsolve(L, R(:,p)\x));
    y = y(q2);
  else
    y = R(p,:) \ cs_ltsolve(L, cs_utsolve(U, x(q)));
  end
end
