function [v, h] = rgs(v, U)

%RGS  Repeated classical Gram-Schmidt
% function [v, h] = rgs(v, U)
% In:  U'U = I
% Out: v = (I-UU')v computed in a stable way
%      h = coefficients u_j'*v, for instance for use in Krylov methods
%
% Revision date: March 7, 2014
% (C) Michiel Hochstenbach 2016

% Defining constant for repetition
c = 0.70; % ~= 1/sqrt(2);

norm_input = sqrt(v'*v);
k = size(U,2);

if k == 0
  h = sqrt(v'*v);
  v = v / h;
  return
end

for l = 1:2
  if l == 2 && (norm_output > c * norm_input)
    return
  end
  a = U'*v;
  v = v-U*a;
  norm_output = sqrt(v'*v);   % Faster than norm
  v = v * (1 / norm_output);  % Faster than division
  if nargout > 1
    if l == 1
      h = [a; 1];
    else
      h(1:k,1) = h(1:k,1) + h(k+1,1) * a;
    end
    h(k+1,1) = h(k+1,1) * norm_output;
  end
end
